"""steamstore.py - A Python wrapper for the Public Steam StoreFront API"""

__version__ = '0.0.4'
__author__ = 'Sauce <saucesteals@gmail.com>'
__all__ = []

from .client import Client
from .app import *


import logging
import asyncio

import aiohttp


from .defaults import *
from .app import App
from .featured import FeaturedList

log = logging.getLogger(__name__)

class Client:
    """Main class for the Steam API"""

    def __init__(self, *, loop=None, **opts):
        self.loop = asyncio.get_event_loop() if loop is None else loop
        self.ready = False
        self.http = None

    def __cleanup(self):
        loop = self.loop

        if self.http:
            asyncio.ensure_future(self.http.close(), loop=loop)

    def stop(self):
        self.__cleanup()

    def __build_api(self, path:str, qs:dict=None, **args):
        url = base_api + path

        if qs:
            url += "?" + '&'.join([name + "=" + str(value) for name, value in qs.items()])

        return url

    async def __get_json(self, req:aiohttp.client_reqrep.ClientResponse, *args, **kwargs) -> dict:
        json_resp = None
        try:
            json_resp = await req.json()
        except:
            print("Error")
            return # TODO: Handle this

        return json_resp

    

    async def start(self, *args, **kwargs):
        
        self.http = await aiohttp.ClientSession().__aenter__()
        self.ready = True

    def run(self, *args, **kwargs):

        loop = self.loop

        async def runner():
            try:
                await self.start(*args, **kwargs)
            finally:
                pass # TODO: Handle this

        asyncio.ensure_future(runner(), loop=loop)
        
        try:
            loop.run_forever()
        except KeyboardInterrupt:
            self.__cleanup()


    async def get_app_from_id(self, appid:int, currency_code:str="us", language_code:str="en"):

        req = await self.http.get(self.__build_api("appdetails", {"appids":appid, "cc":currency_code, "l":language_code}))
        json_resp = await self.__get_json(req)
        for item in json_resp:
            data = json_resp[item]
            return App(data=data["data"]) if data["success"] else None # TODO: Handle This

    async def get_featured(self, *args, **kwargs):

        req = await self.http.get(self.__build_api("featured"))
        json_resp = await self.__get_json(req)
        return FeaturedList(data=json_resp)




class FeaturedItem():
    def __init__(self, *, data):
        self.id = data["id"]
        self.type = data["type"]
        self.name = data["name"]
        self.discounted = data["discounted"]
        self.discount_percent = data["discount_percent"]
        self.original_price = data["original_price"]
        self.final_price = data["final_price"]
        self.currency = data["currency"]
        self.large_capsule_image = data["large_capsule_image"]
        self.small_capsule_image = data["small_capsule_image"]
        self.windows_available = data["windows_available"]
        self.mac_available = data["mac_available"]
        self.linux_available = data["linux_available"]
        self.streamingvideo_available = data["streamingvideo_available"]
        self.discount_expiration = data["discount_expiration"] if "discount_expiration" in data else None
        self.header_image = data["header_image"]
        self.controller_support = data["controller_support"] if "controller_support" in data else None

class FeaturedList():
    def __init__(self, *, data):
        self.large_capsules = [FeaturedItem(data=item) for item in data["large_capsules"]]
        self.featured_win = [FeaturedItem(data=item) for item in data["featured_win"]]
        self.featured_mac = [FeaturedItem(data=item) for item in data["featured_mac"]]
        self.featured_linux = [FeaturedItem(data=item) for item in data["featured_linux"]]
        self.layout = data["layout"]
        self.status = data["status"]
